//Подключаем библиотеку I2C.
//!!!  Если бибилиотеки нет Установите её из архива Wire.zip
#include <Wire.h>
#include <HMC5883L.h>
// Подключаем библиотеку для создания дополнительных последовательных (Serial) портов.
#include <SoftwareSerial.h>
// Подключаем библиотеку управления сервомоторами.
#include <Servo.h>
// Подключаем библиотеку управляющую дальномером.
#include "Ultrasonic.h"
// Инициализируем дальномер
// Контакт Trig соединен с 5 портом, Echo с 6 портом.
Ultrasonic sonar(5, 6);
// Создаем сервомотор поворта головы.
Servo neck;
// Объявляем переменные для хранения состояния двух моторов.
int motor_L1, motor_L2, input_L;
int motor_R1, motor_R2, input_R;
// Константы - постоянные значения для уточнения углов.
const int left_ang = 168;
const int front_ang = 98;
const int right_ang = 28;
// Временные константы служат для точного задания времени на поворот, разворот, движение вперед
// в миллисекундах.
const int time_90 = 390;
const int time_180 = 750;
const int time_10cm = 220;
// Номера портов к которым подключены датчики препятствия.
const int  Front1 = 2, Front2 = 12, Right = 13;

// Создан объект compass для обращения к устройству HMC5883L.
HMC5883L compass;
/////////////

// Функция инициализации управления моторами.
void setup_motor_system(int L1, int  L2, int iL, int  R1, int R2, int iR)
{
  // Заносим в переменные номера контактов (пинов) Arduino.
  motor_L1 = L1; motor_L2 = L2; input_L = iL;
  // Для левых и правых моторов машинки.
  motor_R1 = R1; motor_R2 = R2; input_R = iR;
  // Переводим указанные порты в состояние вывода данных.
  pinMode(motor_L1, OUTPUT);
  pinMode(motor_L2, OUTPUT);
  pinMode(input_L, OUTPUT);
  pinMode(motor_R1, OUTPUT);
  pinMode(motor_R2, OUTPUT);
  pinMode(input_R, OUTPUT);
}

// Функция задает скорость двигателя.
void setspeed(int LeftSpeed, int RightSpeed)
{
  // Задаем ширину положительного фронта от 0 до 255.
  analogWrite(input_L, LeftSpeed);
  analogWrite(input_R, RightSpeed);
  // Чем больше, тем интенсивнее работает мотор.
}
// движение вперед.
void forward()
{
  // Если двигатель будет работать не в ту сторону,
  // поменять на нем контакты местами.
  digitalWrite(motor_L1, HIGH);
  digitalWrite(motor_L2, LOW);
  digitalWrite(motor_R1, HIGH);
  digitalWrite(motor_R2, LOW);
}
// Поворот налево с блокировкой левых колес.
void forward_left()
{
  // блокируем вращение левых колес.
  digitalWrite(motor_L1, LOW);
  digitalWrite(motor_L2, LOW);
  // правые колеса вращаются.
  digitalWrite(motor_R1, HIGH);
  digitalWrite(motor_R2, LOW);
}
// Поворот налево.
void left()
{
  // левые колеса вращаются назад
  digitalWrite(motor_L1, LOW);
  digitalWrite(motor_L2, HIGH);
  // правые колеса вращаются.
  digitalWrite(motor_R1, HIGH);
  digitalWrite(motor_R2, LOW);
}
// Поворот направо с блокировкой правых колес.
void forward_right()
{
  // левые колеса вращаются.
  digitalWrite(motor_L1, HIGH);
  digitalWrite(motor_L2, LOW);
  // блокируем вращение правых колес.
  digitalWrite(motor_R1, LOW);
  digitalWrite(motor_R2, LOW);
}
void right()
{
  // левые колеса вращаются.
  digitalWrite(motor_L1, HIGH);
  digitalWrite(motor_L2, LOW);
  //правые колеса вращаются назад.
  digitalWrite(motor_R1, LOW);
  digitalWrite(motor_R2, HIGH);
}
// Включаем движение назад.
void backward()
{
  // Меняем направление вращения двигателей.
  digitalWrite(motor_L2, HIGH);
  digitalWrite(motor_L1, LOW);
  digitalWrite(motor_R2, HIGH);
  digitalWrite(motor_R1, LOW);
}
void backward_left()
{
  // Меняем направление вращения двигателей.
  digitalWrite(motor_L2, LOW);
  digitalWrite(motor_L1, LOW);
  digitalWrite(motor_R2, HIGH);
  digitalWrite(motor_R1, LOW);
}

void backward_right()
{
  // Меняем направление вращения двигателей.
  digitalWrite(motor_L2, HIGH);
  digitalWrite(motor_L1, LOW);
  digitalWrite(motor_R2, LOW);
  digitalWrite(motor_R1, LOW);
}
// Остановка
void _stop()
{
  // Блокируем все колеса.
  digitalWrite(motor_L2, LOW);
  digitalWrite(motor_L1, LOW);
  digitalWrite(motor_R2, LOW);
  digitalWrite(motor_R1, LOW);
}
// Настройка компаса HMC5883L.
void setupHMC5883L() {
  // инициализация HMC5883L, и проверка наличия ошибок.
  int error;
  // чувствительность датчика из диапазона: 0.88, 1.3, 1.9, 2.5, 4.0, 4.7, 5.6, 8.1.
  error = compass.SetScale(0.88);
  // если ошибка, то выводим ее.
  if (error != 0) Serial.println(compass.GetErrorText(error));
  // установка режима измерений как Continuous (продолжительный).
  error = compass.SetMeasurementMode(Measurement_Continuous);
  // если ошибка, то выводим ее.
  if (error != 0) Serial.println(compass.GetErrorText(error));
}
//====== Получение данных от компаса.
float get_Compass_ang() {
  // считываем данные с HMC5883L и рассчитываем  направление.
  // получаем масштабированные элементы с датчика.
  MagnetometerScaled scaled = compass.ReadScaledAxis();
  // высчитываем направление.
  float heading = atan2(scaled.YAxis, scaled.XAxis);
  // корректируем значения с учетом знаков.
  if (heading < 0) heading += 2 * PI;
  if (heading > 2 * PI) heading -= 2 * PI;
  // переводим радианы в градусы.
  return heading * RAD_TO_DEG;
}
//================================================


void setup()
{
  // Инициализируем сервомотор, управление 9 портом.
  //neck.attach(9);
  // Заносим в переменные номера контактов (пинов) Arduino.
  // Для левых и правых моторов машинки.
  setup_motor_system(3, 4, 11, 7, 8, 10);
  // Инициализируем порты датчиков препятствия.
  pinMode(Front1, INPUT);
  pinMode(Front2, INPUT);
  pinMode(Right, INPUT);
  // Двигатели запущены.
  setspeed(255, 255);
  compass = HMC5883L();  // создаем экземпляр HMC5883L библиотеки
  setupHMC5883L();       // инициализация HMC5883L
  // Устанавливаем скорость передачи данных по кабелю.
  // Порт компьютера
  Serial.begin(9600);
}
// Основная программа.
void loop()
{
  // Создаем переменные для хранения трех дистанций - слева, впереди, справа.
  int Dist_left, Dist_front, Dist_right;
  float Compass_ang;

  setspeed(255, 255);
  _stop();
  Compass_ang = get_Compass_ang();
  while ((Compass_ang > 5) && (Compass_ang < 355))
  {
    if (Compass_ang <= 180)
    {
      left(); //Влево.
    }
    else
    {
      right();//Вправо.
    }
    delay(50);
    _stop();
    delay(10);
    Compass_ang = get_Compass_ang();
  }
  // Анализ обстановки на наличие препятствий.
  // Поворот головы налево.
  neck.write(left_ang);
  // Ждет, так как поворот занимает небольшое время.
  delay(150);
  // Записывает расстояние до препятствия слева.
  Dist_left = sonar.Ranging(CM);
  // Поворачивает голову прямо вперед.
  neck.write(front_ang);
  // Ждет, так как поворот занимает небольшое время.
  delay(150);
  // Записывает расстояние до препятствия впереди.
  Dist_front = sonar.Ranging(CM);
  // Поворачивает голову направо.
  neck.write(right_ang);
  // Ждет, так как поворот занимает небольшое время.
  delay(150);
  // Записывает расстояние до препятствия справа.
  Dist_right = sonar.Ranging(CM);
  neck.write(left_ang);
  if (Dist_front > 20) {
    forward(); // едеn вперед.
  }
  else
  {
    if ((Dist_left > 20) || (Dist_right > 20))
    {
      if (Dist_left > Dist_right)
      {
        left(); // поворачиваем налево 0,5 секунд.
        delay(time_90);
      }
      else
      {
        right(); // поворачиваем направо 0,5 секунд.
        delay(time_90);
        forward(); // едем вперед 0.5 секунды.
        delay(time_10cm * 2);
      }
    }
    else //Заехал в тупик.
    {
      backward();
      delay(time_10cm);
      _stop();
      neck.write(left_ang);
      // Ждет, так как поворот занимает небольшое время.
      delay(150);
      // Записывает расстояние до препятствия слева.
      Dist_left = sonar.Ranging(CM);
      // Поворачивает голову направо.
      neck.write(right_ang);
      // Ждет, так как поворот занимает небольшое время.
      delay(200);
      // Записывает расстояние до препятствия справа.
      Dist_right = sonar.Ranging(CM);
    }
  }
}


